#!/usr/bin/env python

import sys
from optparse import OptionParser
import socket

import pyutilib.pyro
from pyutilib.pyro import Pyro as _pyro


def main():
    parser = OptionParser()
    parser.add_option(
        "--verbose",
        dest="verbose",
        help="Activate verbose output.",
        action="store_true",
        default=False)
    parser.add_option(
        "--max-allowed-connections",
        dest="max_allowed_connections",
        help=("Set the maximum number of proxy connections allowed to "
              "be made to this dispatcher. By default, the environment "
              "variable PYUTILIB_PYRO_MAXCONNECTIONS is used if present; "
              "otherwise, the default settings for Pyro or Pyro4 are used."),
        type="int",
        default=None)
    parser.add_option(
        "--worker-limit",
        dest="worker_limit",
        help=(
            "Set the maximum number of workers allowed to register "
            "with this dispatcher. By default, no limit is enforced. Note that "
            "whether or not this option is set, the maximum number of possible "
            "worker connections might be limited by other default Pyro settings "
            "(see --max-allowed-connections)."),
        type="int",
        default=None)
    parser.add_option(
        "--daemon-host",
        dest="daemon_host",
        metavar="HOST",
        help=("Hostname that the dispatcher daemon should bind on. "
              "The default is the current hostname."),
        default=socket.gethostname())
    parser.add_option(
        "--daemon-port",
        dest="daemon_port",
        metavar="PORT",
        help="Port that the dispatcher daemon should bind on",
        type="int",
        default=0)
    parser.add_option(
        "-n",
        "--host",
        dest="nameserver_host",
        metavar="HOST",
        help="Hostname that the nameserver is bound on",
        default=None)
    parser.add_option(
        "-p",
        "--port",
        dest="nameserver_port",
        metavar="PORT",
        help="Port that the nameserver is bound on",
        type="int",
        default=None)
    parser.add_option(
        "--allow-multiple-dispatchers",
        dest="allow_multiple_dispatchers",
        help="Allow multiple dispatchers to run under the default nameserver group",
        default=False,
        action="store_true")

    options, args = parser.parse_args()
    # Handle the old syntax which was purly argument driven
    # e.g., <nameserver-host> <verbose flag>
    verbose = False
    if len(args) == 2:
        nameserver_host = sys.argv[1]
        if nameserver_host == "None":
            nameserver_host = None
        verbose = bool(sys.argv[2])
        print(
            "DEPRECATION WARNING: dispatch_srvr is now option driven (see dispatch_srvr --help)")
    elif len(args) == 1:
        nameserver_host = sys.argv[1]
        if nameserver_host == "None":
            nameserver_host = None
        print(
            "DEPRECATION WARNING: dispatch_srvr is now option driven (see dispatch_srvr --help)")
    else:
        nameserver_host = options.nameserver_host
        verbose = options.verbose

    if _pyro is None:
        raise ImportError("Pyro or Pyro4 is not available")
    return pyutilib.pyro.DispatcherServer(
        daemon_host=options.daemon_host,
        daemon_port=options.daemon_port,
        nameserver_host=nameserver_host,
        nameserver_port=options.nameserver_port,
        verbose=verbose,
        max_allowed_connections=options.max_allowed_connections,
        worker_limit=options.worker_limit,
        clear_group=not options.allow_multiple_dispatchers)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# This script searches for a local python installation, which is
# applied, rather than the python executable on the user's path.
#
# Additionally, this script sets the PATH environmental variable to
# point to the acro/bin directory.
#

import os
from os.path import abspath, basename, dirname, exists, join
import sys
import signal


def lpython(args):
    sys.tracebacklimit = 0
    process = None

    def signal_handler(signum, frame):
        if not process is None:
            if sys.platform == "win32":
                # On Windows this calls the Win32 API function TerminateProcess()
                process.terminate()
            else:
                process.send_signal(signal.SIGTERM)
        sys.exit(-signum)

    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGHUP, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)

    #
    # Recurse up the current path, looking for a subdirectory that
    # contains 'python/bin/python'
    #
    pexec = None
    try:
        # On unix, the shell's reported PWD may be different from the '.'
        # inode's absolute path (e.g., when the cwd was reached through
        # symbolic links.)  This is a trivial way to defer to the shell's
        # notion of the cwd and quietly fall back on Python's getcwd() for
        # non-unix environments.
        curr = os.environ['PWD']
    except:
        curr = abspath(os.getcwd())
    while os.sep in curr:
        if exists(join(curr, "python", "bin", "python")):
            pexec = join(curr, "python", "bin", "python")
            os.environ["PATH"] = join(curr,
                                      "bin") + os.pathsep + os.environ["PATH"]
            break
        if exists(join(curr, "python", "bin", "python.exe")):
            pexec = join(curr, "python", "bin", "python.exe")
            os.environ["PATH"] = join(curr,
                                      "bin") + os.pathsep + os.environ["PATH"]
            break
        if exists(join(curr, "bin", "python")):
            pexec = join(curr, "bin", "python")
            break
        if exists(join(curr, "bin", "python.exe")):
            pexec = join(curr, "bin", "python.exe")
            break
        if basename(curr) == "":
            break
        curr = dirname(curr)

    if pexec is None:
        pexec = 'python'
    os.execvp(pexec, [pexec] + args)

    #try:
    #    import subprocess
    #    process = subprocess.Popen(args)
    #    process.wait()
    #    return process.returncode
    #
    #except ImportError:
    #    #
    #    # For Python 2.3
    #    #
    #    if pexec is None:
    #        returncode = os.system(" ".join(["python"]+args))
    #    else:
    #        returncode = os.system(" ".join([pexec]+args))
    #    return returncode


# The [console_scripts] entry point requires a function that takes no
# arguments
def main():
    sys.exit(lpython(sys.argv[1:]))

if __name__ == '__main__':
    main()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import os
import imp
import sys

import pyutilib.common

try:
    import runpy
    runpy_available = True
except ImportError:  #pragma:nocover
    try:
        import runpy2 as runpy
        runpy_available = True
    except ImportError:  #pragma:nocover
        runpy_available = False


def import_file(filename, context=None, name=None, clear_cache=False):
    """
    Import a Python file as a module

    The filename argument can be a module name or a python script
    (.py, .pyc).  The module name will be extracted from the filename
    argument by removing any directory components along with the
    .py (or .pyc) suffix if any of these exist. If the module name
    already appears in sys.modules, the corresponding module entry
    will be returned from this dictionary (unless clear_cache is
    True). Otherwise, an attempt will be made to import the module
    name extracted from the filename argument (using any directory
    information that was included).

    The name keyword can be used to always force reloading of the
    module from scratch. The module, if succesully imported, will be
    assigned the supplied name in sys.modules. This option is not
    currently supported for module names that contain one or more
    periods that indicate a submodule.

    This function returns the module object that is created.
    """

    #
    # Parse the filename to get the name of the module to be imported
    # and determine if it contains any directory information about
    # where to find the module.
    #
    dirname = os.path.dirname(filename)
    implied_dirname = None
    if dirname == '':
        dirname = None
        implied_dirname = os.path.dirname(os.path.abspath(filename))
    modulename = os.path.basename(filename)
    is_file = False
    # NB: endswith accepts tuples of strings starting in python 2.5.
    # For 2.4 compatibility we will call endswith() twice.
    if modulename.endswith('.py') or modulename.endswith('.pyc'):
        if not os.path.exists(filename):
            if not os.path.exists(os.path.expanduser(filename)):
                raise IOError("File %s does not exist" % (filename))
            filename = os.path.expanduser(filename)
        if filename.endswith('.pyc'):
            filename = filename[:-1]
        modulename = modulename.rsplit('.', 1)[0]
        is_file = True

    module = None
    # If the name keyword is not used we will first check if the
    # module name (as extracted from filename) already exists
    # sys.modules. Only if it does not already exist in sys.modules do
    # we try to import.
    # **NOTE**: This will ignore any directory specifications that
    #           were included in the filename, meaning
    #              filename=foo/bar.py
    #           would be overlooked if someone previously imported any
    #           module 'bar' from a different source. As a workaround
    #           for this issue, the 'name' keyword was added to this
    #           function, which indicates that the module source
    #           should be located, loaded from scratch, and assigned
    #           the module name specified by this keyword.
    if name is None:
        name = modulename
        if not clear_cache and modulename in sys.modules:
            module = sys.modules[modulename]
        else:
            if clear_cache and modulename in sys.modules:
                del sys.modules[modulename]
            if dirname is not None:
                sys.path.insert(0, dirname)
            else:
                sys.path.insert(0, implied_dirname)
            try:
                module = __import__(modulename)
            except ImportError:
                pass
            finally:
                if dirname is not None:
                    sys.path.remove(dirname)
                else:
                    sys.path.remove(implied_dirname)
            # This extra assignment seems redundant, but it is necessary
            # when the module name contains one or more periods to
            # indicate a submodule, in which case the __import__
            # function only returns the top level module. The true
            # submodule must be located inside the sys.modules dict.
            if modulename in sys.modules:
                module = sys.modules[modulename]

    # If we are entering this next block, it means that the optional
    # name keyword was used or that we encountered an issue with the
    # __import__ function (which can happen if the true filename on
    # disk contained periods in its name, e.g., file_1.0.py)
    if module is None:
        pathname = None
        if is_file:
            # .py or .pyc were found in the filename
            pathname = filename
        else:
            if dirname is not None:
                # find_module will return the .py file (never .pyc)
                fp, pathname, description = imp.find_module(modulename,
                                                            [dirname])
                fp.close()
            else:
                sys.path.insert(0, implied_dirname)
                try:
                    # find_module will return the .py file
                    # (never .pyc)
                    fp, pathname, description = imp.find_module(modulename)
                    fp.close()
                except ImportError:
                    raise
                finally:
                    sys.path.remove(implied_dirname)
        try:
            # Note: we are always handing load_source a .py file, but
            #       it will use the .pyc or .pyo file if it exists
            module = imp.load_source(name, pathname)
        except:
            et, e, tb = sys.exc_info()
            import traceback
            _, line, _, txt = traceback.extract_tb(tb, 2)[-1]
            import logging
            logger = logging.getLogger('pyutilib.misc')
            msg = ''
            if isinstance(e, Exception):
                msg = " raised %s:\n%s" % (et.__name__, e)
            logger.error('Failed to load python module "%s"\n'
                         '%s line %s ("%s")%s' %
                         (filename, pathname, line, txt, msg))
            raise

    #
    # Add module to the given context
    #
    if not context is None:
        context[name] = module
    return module


def run_file(filename, logfile=None, execdir=None):
    """
    Execute a Python file and optionally redirect output to a logfile.
    """
    if not runpy_available:  #pragma:nocover
        raise pyutilib.common.ConfigurationError(
            "Cannot apply the run_file() function because runpy is not available")
    #
    # Open logfile
    #
    if not logfile is None:
        sys.stderr.flush()
        sys.stdout.flush()
        save_stdout = sys.stdout
        save_stderr = sys.stderr
        OUTPUT = open(logfile, "w")
        sys.stdout = OUTPUT
        sys.stderr = OUTPUT
    #
    # Add the file directory to the system path
    #
    currdir_ = ''
    if '/' in filename:
        currdir_ = "/".join((filename).split("/")[:-1])
        tmp_import = (filename).split("/")[-1]
        sys.path.append(currdir_)
    elif '\\' in filename:
        currdir_ = "\\".join((filename).split("\\")[:-1])
        tmp_import = (filename).split("\\")[-1]
        sys.path.append(currdir_)
    else:
        tmp_import = filename
    name = ".".join((tmp_import).split(".")[:-1])
    #
    # Run the module
    #
    try:
        if not execdir is None:
            tmp = os.getcwd()
            os.chdir(execdir)
            tmp_path = sys.path
            sys.path = [execdir] + sys.path
        runpy.run_module(name, None, "__main__")
        if not execdir is None:
            os.chdir(tmp)
            sys.path = tmp_path
    except Exception:  #pragma:nocover
        if not logfile is None:
            OUTPUT.close()
            sys.stdout = save_stdout
            sys.stderr = save_stderr
        raise
    if currdir_ in sys.path:
        sys.path.remove(currdir_)
    if execdir in sys.path:
        sys.path.remove(execdir)
    #
    # Close logfile
    #
    if not logfile is None:
        OUTPUT.close()
        sys.stdout = save_stdout
        sys.stderr = save_stderr



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Calculates the total number of downloads that a particular PyPI package has
received across all versions tracked by PyPI
"""

try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict
from datetime import datetime
import locale
import re
import sys
import re
try:
    import xmlrpc.client as xmlrpc
except ImportError:
    import xmlrpc
import math
from optparse import OptionParser

total_packages = 0
total_downloads = 0
total_releases = 0

starttime = None

locale.setlocale(locale.LC_ALL, '')


def total_seconds(td):
    return (td.microseconds +
            (td.seconds + td.days * 24 * 3600) * 10**6) / (1.0 * 10**6)


class PyPIDownloadAggregator(object):

    def __init__(self, package_name, include_hidden=True, exact=False):
        self.package_name = package_name
        self.include_hidden = include_hidden
        self.proxy = xmlrpc.Server('http://pypi.python.org/pypi')
        self._downloads = {}
        self._first_upload = {}
        self._last_upload = {}
        self._exact = exact

        self.first_upload = None
        self.first_upload_rel = None
        self.last_upload = None
        self.last_upload_rel = None

    def packages(self):
        result = self.proxy.package_releases(self.package_name,
                                             self.include_hidden)
        # no matching package--search for possibles
        results = self.proxy.search({
            'name': self.package_name,
            'description': self.package_name
        }, 'or')

        # make sure we only get unique package names
        matches = []
        for match in results:
            if self._exact:
                name = match['name']
                if name not in matches:
                    matches.append(name)
            elif match['name'].startswith(self.package_name):
                matches.append(match)
        return matches

    def downloads(self, release, force=False):
        """Calculate the total number of downloads for the package"""

        urls = self.proxy.release_urls(release['name'], release['version'])
        if not release['name'] in self._downloads:
            self._downloads[release['name']] = OrderedDict()
        if not release['name'] in self._first_upload:
            self._first_upload[release['name']] = OrderedDict()
        if not release['name'] in self._last_upload:
            self._last_upload[release['name']] = OrderedDict()
        self._downloads[release['name']][release['version']] = 0

        for url in urls:
            # upload times
            uptime = datetime.strptime(url['upload_time'].value,
                                       "%Y%m%dT%H:%M:%S")
            #if not starttime is None and uptime <= starttime:
            #    continue

            if self._first_upload[release['name']].get(
                    release['version'],
                    None) is None or uptime < self._first_upload[release[
                        'name']][release['version']]:
                self._first_upload[release['name']][release['version']] = uptime
            if self._last_upload[release['name']].get(
                    release['version'],
                    None) is None or uptime > self._last_upload[release[
                        'name']][release['version']]:
                self._last_upload[release['name']][release['version']] = uptime

            self._downloads[release['name']][release['version']] += url[
                'downloads']

    def stats(self):
        """Prints a nicely formatted list of statistics about the package"""

        print("")
        for release in self.packages():
            print("Processing ... %s %s" %
                  (release['name'], release['version']))
            self.downloads(release)

        for pkg in sorted(self._downloads.keys()):
            global total_packages
            global total_downloads
            global total_releases

            print("")

            ndownloads = 0
            npackages = 0
            keys = self._downloads[pkg].keys()

            def keygen(x):
                ans = []
                for v in x.split('.'):
                    m = re.match('([0-9]+)(.*)', v)
                    ans.append(m.groups())
                return ans
            #keys = [k for k in self._downloads[pkg].keys() if not 'rc' in k]
            #keys.sort(key=lambda x: [int(y) for y in x.split('.')])
            keys.sort(key=keygen)
            for i in range(len(keys)):
                key = keys[i]
                if key in self._first_upload[pkg]:
                    if starttime is not None:
                        if i + 1 < len(keys) and self._first_upload[pkg][keys[
                                i + 1]] <= starttime:
                            continue
                        if i + 1 < len(keys) and self._first_upload[pkg][
                                key] <= starttime:
                            ntimediff = self._first_upload[pkg][keys[
                                i + 1]] - starttime
                            dtimediff = self._first_upload[pkg][keys[
                                i + 1]] - self._first_upload[pkg][keys[i]]
                            downloads = int(
                                math.floor(self._downloads[pkg][
                                    key] * total_seconds(ntimediff) /
                                           total_seconds(dtimediff)))
                        else:
                            downloads = self._downloads[pkg][key]
                    else:
                        downloads = self._downloads[pkg][key]
                    #print downloads, self._downloads[pkg][key]
                    ndownloads += downloads
                    npackages += 1
                    print(
                        "Package %s  Release %10s  Downloads %10d  First Upload %25s  Last Upload %25s"
                        % (pkg, key, downloads, self._first_upload[pkg][key],
                           self._last_upload[pkg][key]))
            print("""Totals:  Package %35s  Downloads: %15s  Releases: %15s""" %
                  (pkg, str(ndownloads), str(npackages)))

            total_packages += 1
            total_downloads += ndownloads
            total_releases += npackages


def _main(argv, exact):
    if len(argv) < 2:
        print(
            "Usage: pypi_downloads.py [--start=yyyy-mm-dd] [--exact] package ...")
        sys.exit('Please specify at least one package name')

    for pkg in argv[1:]:
        PyPIDownloadAggregator(pkg, exact=exact).stats()

    print("")
    print("Total Packages %d" % total_packages)
    print("Total Downloads %d" % total_downloads)
    print("Total Releases %d" % total_releases)


def main():
    global starttime

    parser = OptionParser()
    parser.add_option('--start', action="store", dest="date", default=None)
    parser.add_option(
        '--exact', action="store_false", dest="exact", default=False)
    (options, args) = parser.parse_args(sys.argv)
    if not options.date is None:
        starttime = datetime.strptime(options.date, "%Y-%m-%d")
    else:
        starttime = None
    _main(args, options.exact)


if __name__ == '__main__':
    main()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import sys
try:
    from io import StringIO
except:
    from StringIO import StringIO


class TeeStream(object):
    """This class implements a simple 'Tee' of the specified python
    stream.  Since this presents a full file interface, TeeStream
    objects may be arbitrarily nested."""

    def __init__(self, stream):
        self.stream = stream
        self.buffer = StringIO()

    def write(self, data):
        self.buffer.write(data)
        self.stream.write(data)

    def writelines(self, sequence):
        for x in sequence:
            self.write(x)

    def reset(self):
        self.buffer = StringIO()


class ConsoleBuffer(object):
    """This class implements a simple 'Tee' of the python stdout and
    stderr so the output can be captured and reported programmatically.
    We need a specialized class here because other applications /
    methods explicitly write to sys.stderr and sys.stdout, so it is
    insufficient to 'wrap' the streams like the TeeStream class;
    instead, we must replace the standard stdout and stderr objects with
    our own duplicator."""

    class _Duplicate(object):

        def __init__(self, a, b):
            self.a = a
            self.b = b

        def write(self, data):
            self.a.write(data)
            self.b.write(data)

    def __init__(self):
        self._dup_out = self._dup_err = None
        self._raw_out = sys.stdout
        self._raw_err = sys.stderr
        self.reset()

    def __del__(self):
        if self._dup_out is not None and self._dup_out is not sys.stdout:
            raise RuntimeError("ConsoleBuffer: Nesting violation " \
                  "(attempting to delete the buffer while stdout is " \
                  "redirected away from this buffer).")
        if self._dup_err is not None and self._dup_err is not sys.stderr:
            raise RuntimeError("ConsoleBuffer: Nesting violation " \
                  "(attempting to delete the buffer while stderr is " \
                  "redirected away from this buffer).")
        sys.stdout = self._raw_out
        sys.stderr = self._raw_err

    def reset(self):
        if self._dup_out is not None and self._dup_out is not sys.stdout:
            raise RuntimeError("ConsoleBuffer: Nesting violation " \
                  "(attempting to reset() when stdout has been redirected " \
                  "away from this buffer).")
        if self._dup_err is not None and self._dup_err is not sys.stderr:
            raise RuntimeError("ConsoleBuffer: Nesting violation " \
                  "(attempting to reset() when stderr has been redirected " \
                  "away from this buffer).")

        self.out = StringIO()
        self.err = StringIO()
        self._dup_out = sys.stdout = \
                        ConsoleBuffer._Duplicate(self.out, self._raw_out)
        self._dup_err = sys.stderr = \
                        ConsoleBuffer._Duplicate(self.err, self._raw_err)



import pyutilib.component.app
import os
import sys

currdir = sys.argv[-1] + os.sep

app = pyutilib.component.app.SimpleApplication("foo")
app._env_config.options.path = "DEFAULT PATH HERE"
app.config.save(currdir + "config1.out")



from collections import deque

class SimpleVisitor(object):

    def visit(self, node):
        pass

    def children(self, node):
        return node.children
    
    def is_leaf(self, node):
        return len(node.children) == 0

    def finalize(self):
        pass

    def bfs(self, node):
        """
        Breadth-first search
        """
        dq = deque([node])
        while dq:
            current = dq.popleft()
            self.visit(current)
            if not self.is_leaf(current):
                dq.extend(self.children(current))
        return self.finalize()

    def xbfs(self, node):
        """
        Breadth-first search, except that 
        leaf nodes are immediately visited.
        """
        dq = deque([node])
        while dq:
            current = dq.popleft()
            self.visit(current)
            for c in self.children(current):
                if self.is_leaf(c):
                    self.visit(c)
                else:
                    dq.append(c)
        return self.finalize()

    def dfs_preorder(self, node):
        """
        Depth-first search - preorder
        """
        dq = deque([node])
        while dq:
            current = dq.pop()
            self.visit(current)
            if not self.is_leaf(current):
                for c in reversed(self.children(current)):
                    dq.append(c)
        return self.finalize()

    dfs = dfs_preorder

    def dfs_postorder(self, node):
        """
        Depth-first search - postorder
        """
        expanded = set()
        dq = deque([node])
        while dq:
            current = dq[-1]
            if id(current) in expanded:
                dq.pop()
                self.visit(current)
            else:
                for c in reversed(self.children(current)):
                    dq.append(c)
                expanded.add(id(current))
        return self.finalize()

    def dfs_inorder(self, node):
        """
        Depth-first search - inorder
        """
        expanded = set()
        dq = deque([node])
        while dq:
            current = dq.pop()
            if id(current) in expanded or self.is_leaf(current):
                self.visit(current)
            else:
                first = True
                for c in reversed(self.children(current)):
                    if first:
                        first = False
                    else:
                        dq.append(current)
                    dq.append(c)
                expanded.add(id(current))
        return self.finalize()


class ValueVisitor(object):

    def visit(self, node, values):
        pass

    def children(self, node):
        return node.children
    
    def is_leaf(self, node):
        return len(node.children) == 0

    def finalize(self, ans):
        pass

    def visiting_potential_leaf(self, node, values):
        """ 
        Visiting a potential leaf.

        Return True if the node is not expanded.
        """
        if not self.is_leaf(node):
            return False
        ans = self.visit(node, None)
        if ans is None:
            return False
        values.append( ans )
        return True

    def dfs_postorder_deque(self, node):
        """
        Depth-first search - postorder with a dequeue
        """
        _values = [[]]
        expanded = set()
        dq = deque([node])
        while dq:
            current = dq[-1]
            if id(current) in expanded:
                dq.pop()
                values = _values.pop()
                _values[-1].append( self.visit(current, values) )
            elif self.visiting_potential_leaf(current, _values[-1]):
                dq.pop()
            else:
                for c in reversed(self.children(current)):
                    dq.append(c)
                expanded.add(id(current))
                _values.append( [] )
        return self.finalize(_values[-1][0])

    def dfs_postorder_stack(self, node):
        """
        Depth-first search - postorder with a stack
        """
        _stack = [ (node, self.children(node), 0, len(self.children(node)), [])]
        #
        # Iterate until the stack is empty
        #
        # Note: 1 is faster than True for Python 2.x
        #
        while 1:
            #
            # Get the top of the stack
            #   _obj        Current expression object
            #   _argList    The arguments for this expression objet
            #   _idx        The current argument being considered
            #   _len        The number of arguments
            #
            _obj, _argList, _idx, _len, _result = _stack.pop()
            #
            # Iterate through the arguments
            #
            while _idx < _len:
                _sub = _argList[_idx]
                _idx += 1
                if not self.visiting_potential_leaf(_sub, _result):
                    #
                    # Push an expression onto the stack
                    #
                    _stack.append( (_obj, _argList, _idx, _len, _result) )
                    _obj                    = _sub
                    _argList                = self.children(_sub)
                    _idx                    = 0
                    _len                    = len(_argList)
                    _result                 = []
            #
            # Process the current node
            #
            ans = self.visit(_obj, _result)
            if _stack:
                #
                # "return" the recursion by putting the return value on the end of the results stack
                #
                _stack[-1][-1].append( ans )
            else:
                return self.finalize(ans)




#
# Unit Tests for util/math
#
#

from os.path import abspath, dirname
pkgdir = dirname(abspath(__file__)) + "/../.."

import pyutilib.th as unittest
import pyutilib.misc


class CrossDebug(unittest.TestCase):

    def setUp(self):
        self.tmp = [(10, 22, 31), (10, 22, 32), (10, 22, 33), (10, 23, 31),
                    (10, 23, 32), (10, 23, 33), (11, 22, 31), (11, 22, 32),
                    (11, 22, 33), (11, 23, 31), (11, 23, 32), (11, 23, 33)]
        self.tmp.sort()

        self.ttmp = [(10, 22, 31), (10, 23, 32), (11, 22, 31), (11, 23, 32)]
        self.ttmp.sort()

    def test_cross1(self):
        # Apply the cross() method
        ans = pyutilib.misc.cross(((10, 11), (22, 23), (31, 32, 33)))
        ans.sort()
        self.assertEqual(ans, self.tmp)

    def test_cross2(self):
        # Apply the cross_iter() method
        ans = []
        for item in pyutilib.misc.cross_iter((10, 11), (22, 23), (31, 32, 33)):
            ans.append(item)
        ans.sort()
        self.assertEqual(ans, self.tmp)

    def test_cross3(self):
        # Apply the flattened_cross_iter() method
        ans = []
        for item in pyutilib.misc.flattened_cross_iter((10, 11), ((22, 31),
                                                                  (23, 32))):
            ans.append(item)
        ans.sort()
        self.assertEqual(ans, self.ttmp)


if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import pyutilib.component.core
pyutilib.component.core.PluginGlobals.add_env("pyutilib.workflow")

from pyutilib.workflow.resource import Resource
from pyutilib.workflow.task import Task, EmptyTask, Component, Port, Ports, InputPorts, OutputPorts, Connector
from pyutilib.workflow.workflow import Workflow
from pyutilib.workflow.file import FileResource
from pyutilib.workflow.executable import ExecutableResource
from pyutilib.workflow.tasks import TaskPlugin, TaskFactory, WorkflowPlugin
from pyutilib.workflow.driver import TaskDriver
from pyutilib.workflow.functor import functor_api, IFunctorTask, FunctorAPIFactory, FunctorAPIData

pyutilib.component.core.PluginGlobals.pop_env()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

__all__ = ['ExcelSpreadsheet']

import pyutilib.common

#
# Attempt to import openpyxl
#
try:
    import openpyxl
    _openpyxl = True
except:
    _openpyxl = False
#
# Attempt to import xlrd
#
try:
    import xlrd
    _xlrd = True
except:
    _xlrd = False
#
# Attempt to import win32com stuff. 
#
try:
    from win32com.client.dynamic import Dispatch
    from pythoncom import CoInitialize, CoUninitialize
    from pythoncom import CoInitialize, CoUninitialize, com_error
    _win32com = True
except:
    _win32com = False  #pragma:nocover

from pyutilib.excel.base import ExcelSpreadsheet_base

if _win32com:
    from pyutilib.excel.spreadsheet_win32com import ExcelSpreadsheet_win32com
else:

    class ExcelSpreadsheet_win32com(ExcelSpreadsheet_base):
        pass


if _openpyxl:
    from pyutilib.excel.spreadsheet_openpyxl import ExcelSpreadsheet_openpyxl
else:

    class ExcelSpreadsheet_openpyxl(ExcelSpreadsheet_base):
        pass


if _xlrd:
    from pyutilib.excel.spreadsheet_xlrd import ExcelSpreadsheet_xlrd
else:

    class ExcelSpreadsheet_xlrd(ExcelSpreadsheet_base):
        pass


class ExcelSpreadsheet(ExcelSpreadsheet_base):

    def __new__(cls, *args, **kwds):
        #
        # Note that this class returns class instances rather than
        # class types.  This is because these classes are not
        # subclasses of ExcelSpreadsheet, and thus the __init__
        # method will not be called unless we construct the
        # class instances here.
        #
        ctype = kwds.pop('ctype', None)
        if ctype == 'xlrd':
            return ExcelSpreadsheet_xlrd(*args, **kwds)
        if ctype == 'win32com':
            return ExcelSpreadsheet_win32com(*args, **kwds)
        if ctype == 'openpyxl':
            return ExcelSpreadsheet_openpyxl(*args, **kwds)
        #
        if _xlrd:
            return ExcelSpreadsheet_xlrd(*args, **kwds)
        if _win32com:
            return ExcelSpreadsheet_win32com(*args, **kwds)
        if _openpyxl:
            return ExcelSpreadsheet_openpyxl(*args, **kwds)
        #
        return super(ExcelSpreadsheet, cls).__new__(cls)



#!/usr/bin/env python
#
# A script that replaces copyright statements in files with a
# new copyright statement
#
# replaceCopyright old new file [file...]
#

import sys
import os


def match_head(file, lines):
    INPUT = open(file)
    i = 0
    for line in INPUT:
        if line.strip() != lines[i].strip():
            return False
        i = i + 1
        if i == len(lines):
            break
    return True


def main():
    INPUT = open(sys.argv[1])
    old_cr = INPUT.readlines()
    INPUT.close()
    for i in range(0, len(old_cr)):
        old_cr[i] = old_cr[i].strip()

    INPUT = open(sys.argv[2])
    new_cr = INPUT.readlines()
    INPUT.close()

    #print old_cr
    #print ""
    #print new_cr

    for file in sys.argv[3:]:
        INPUT = open(file)
        OUTPUT = open(file + ".tmp", "w")
        i = 0
        newfile = True
        for line in INPUT:
            if i == len(old_cr):
                #
                # Print new copyright
                #
                for crline in new_cr:
                    OUTPUT.write(crline)
                i = i + 1
            if i > len(old_cr):
                #
                # Print lines from the old file
                #
                OUTPUT.write(line)
            else:
                #
                # Keep checking that the copyright is what is expected
                #
                if line.strip() == old_cr[i]:
                    i = i + 1
                else:
                    INPUT.close()
                    if match_head(file, new_cr):
                        print("File %s is up to date." % file)
                    else:
                        print("Unexpected line in file %s\n" % file)
                        print("  Expected line:\n")
                        print(old_cr[i] + '\n')
                        print("  Current line:\n")
                        print(line)
                    newfile = False
                    break
        OUTPUT.close()
        INPUT.close()
        if newfile:
            os.remove(file)
            os.rename(file + ".tmp", file)
            print("Updating file %s\n" % file)
        else:
            os.remove(file + ".tmp")


if __name__ == '__main__':
    main()
